// RegDocMgr.cpp: implementation of the CRegDocMgr class.
//
//////////////////////////////////////////////////////////////////////
// define _HIDEALLFILTER to remove the *.* from the file dialog

// If you want to get to the MRUD in your code, say for example, an export function,
// use this code when initializing the import/export dialog.
/*
	// setup for the EXPORT file dialog

	// for a probable title
	CRDMSampleDoc* pDoc = GetDocument();
	CString csTitle = pDoc->GetTitle();
	if (csTitle != "Untitled")
		csTitle = csTitle.Left(csTitle.GetLength() - 4);

	// file filters
	static char szFilter[] = "ABC Files (*.abc)|*.abc; *.abc|All Files (*.*)|*.*||";

	// create the dialog
	CFileDialog dlgExport(false, "abc", (LPCTSTR) csTitle,
			OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, szFilter, AfxGetMainWnd());
	dlgExport.m_ofn.lpstrTitle = "Export to file"; // window title

	//	IMPORT
	//	static char szFilter[] = "ABC Files (*.abc)|*.abc; *.abc||";
	//	CFileDialog dlgImport(true, "abc", NULL, // no preset filename
	//			OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST, szFilter, AfxGetMainWnd());

	////////////////////////////////////////////////////////////////
	// set a starting directory if one exists
	CRegDocMgr* pDocMgr = (CRegDocMgr*) AfxGetApp()->m_pDocManager;
	dlgExport.m_ofn.lpstrInitialDir = pDocMgr->GetCurrentMRUD();
	////////////////////////////////////////////////////////////////

	// show it
	if (dlgExport.DoModal() == IDOK)
	{
		////////////////////////////////////////////////////////////////
		// omit this if import/export should not affect document MRUD
		pDocMgr->SetMRUD(dlgExport.m_ofn.lpstrFile);
		////////////////////////////////////////////////////////////////

		//... proceed with the import/export ... etc
*/

#include <stdafx.h>
#include <afxwin.h>
#include <afxdlgs.h>
#include "RegDocMgr.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

static void AppendFilterSuffix(CString& filter, OPENFILENAME& ofn,
	CDocTemplate* pTemplate, CString* pstrDefaultExt)
{
	ASSERT_VALID(pTemplate);
	ASSERT_KINDOF(CDocTemplate, pTemplate);

	CString strFilterExt, strFilterName;
	if (pTemplate->GetDocString(strFilterExt, CDocTemplate::filterExt) &&
	 !strFilterExt.IsEmpty() &&
	 pTemplate->GetDocString(strFilterName, CDocTemplate::filterName) &&
	 !strFilterName.IsEmpty())
	{
		// a file based document template - add to filter list
#ifndef _MAC
		ASSERT(strFilterExt[0] == '.');
#endif
		if (pstrDefaultExt != NULL)
		{
			// set the default extension
#ifndef _MAC
			*pstrDefaultExt = ((LPCTSTR)strFilterExt) + 1;  // skip the '.'
#else
			*pstrDefaultExt = strFilterExt;
#endif
			ofn.lpstrDefExt = (LPTSTR)(LPCTSTR)(*pstrDefaultExt);
			ofn.nFilterIndex = ofn.nMaxCustFilter + 1;  // 1 based number
		}

		// add to filter
		filter += strFilterName;
		ASSERT(!filter.IsEmpty());  // must have a file type name
		filter += (TCHAR)'\0';  // next string please
#ifndef _MAC
		filter += (TCHAR)'*';
#endif
		filter += strFilterExt;
		filter += (TCHAR)'\0';  // next string please
		ofn.nMaxCustFilter++;
	}
}
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
IMPLEMENT_DYNAMIC(CRegDocMgr, CDocManager)

CRegDocMgr::CRegDocMgr()
{
  // get the MRUD from the registry
  m_strInitialMRUD = AfxGetApp()->GetProfileString(_T("Settings"), _T("MRUD"));
  m_strMRUD = m_strInitialMRUD;

  CDocManager::CDocManager();
}

CRegDocMgr::~CRegDocMgr()
{
  // write the most recent path to the registry
  // if it differs from when the program was first opened
  if (m_strMRUD != m_strInitialMRUD)
    AfxGetApp()->WriteProfileString(_T("Settings"), _T("MRUD"), (LPCTSTR) m_strMRUD);

  CDocManager::~CDocManager();
}

CString CRegDocMgr::SetMRUD(CString csFileName)
{
	CString csPreviousMRUD;

#ifndef _MAC
 	m_strMRUD = csFileName.Left(csFileName.ReverseFind('\\') + 1);
#else
 	m_strMRUD = csFileName.Left(csFileName.ReverseFind('/') + 1);
#endif

	return csPreviousMRUD;
}

// lifted from the original member function in CDocManager in docmgr.cpp
BOOL CRegDocMgr::DoPromptFileName(CString& fileName, UINT nIDSTitle, DWORD lFlags, BOOL bOpenFileDialog, CDocTemplate* pTemplate)
{
	CFileDialog dlgFile(bOpenFileDialog);

	CString title;
	VERIFY(title.LoadString(nIDSTitle));

	dlgFile.m_ofn.Flags |= lFlags;

  ///////////////////////////////////////////////////
  // THIS IS NOT IN ORIGINAL FUNCTION
  // set a starting directory if one exists
  if (m_strMRUD.IsEmpty())
    dlgFile.m_ofn.lpstrInitialDir = NULL;
  else
    dlgFile.m_ofn.lpstrInitialDir = (LPCTSTR) m_strMRUD;
  ///////////////////////////////////////////////////

  CString strFilter;
	CString strDefault;
	if (pTemplate != NULL)
	{
		ASSERT_VALID(pTemplate);
		AppendFilterSuffix(strFilter, dlgFile.m_ofn, pTemplate, &strDefault);
	}
	else
	{
		// do for all doc template
		POSITION pos = m_templateList.GetHeadPosition();
		BOOL bFirst = TRUE;
		while (pos != NULL)
		{
			CDocTemplate* pTemplate = (CDocTemplate*)m_templateList.GetNext(pos);
			AppendFilterSuffix(strFilter, dlgFile.m_ofn, pTemplate,
				bFirst ? &strDefault : NULL);
			bFirst = FALSE;
		}
	}

///////////////////////////////////////////////////
// THIS ifndef IS NOT IN ORIGINAL FUNCTION
#ifndef _HIDEALLFILTER
  // append the "*.*" all files filter
	CString allFilter;
	VERIFY(allFilter.LoadString(AFX_IDS_ALLFILTER));
	strFilter += allFilter;
	strFilter += (TCHAR)'\0';   // next string please
#ifndef _MAC
	strFilter += _T("*.*");
#else
	strFilter += _T("****");
#endif
	strFilter += (TCHAR)'\0';   // last string
	dlgFile.m_ofn.nMaxCustFilter++;
#endif	// ifndef _HIDEALLFILTER

	dlgFile.m_ofn.lpstrFilter = strFilter;
#ifndef _MAC
	dlgFile.m_ofn.lpstrTitle = title;
#else
	dlgFile.m_ofn.lpstrPrompt = title;
#endif
	dlgFile.m_ofn.lpstrFile = fileName.GetBuffer(_MAX_PATH);


	BOOL bResult = dlgFile.DoModal() == IDOK ? TRUE : FALSE;
	fileName.ReleaseBuffer();

  ///////////////////////////////////////////////////
  // THIS IS NOT IN ORIGINAL FUNCTION
  // update the MRUD variable
  if (bResult)
#ifndef _MAC
    m_strMRUD = fileName.Left(fileName.ReverseFind('\\') + 1);
#else
    m_strMRUD = fileName.Left(fileName.ReverseFind('/') + 1);
#endif
    ///////////////////////////////////////////////////

	return bResult;
}

CDocument* CRegDocMgr::OpenDocumentFile(LPCTSTR lpszFileName)
{
  // this handles the case of the MRU files
  CString fileName(lpszFileName);
#ifndef _MAC
  m_strMRUD = fileName.Left(fileName.ReverseFind('\\') + 1);
#else
  m_strMRUD = fileName.Left(fileName.ReverseFind('/') + 1);
#endif

  return CDocManager::OpenDocumentFile(lpszFileName);
}